/** Rill v0.0.27 https://www.npmjs.com/package/rill License: MIT*/
var Application, Request, Response, chain, co, compose, hijack, http, parseDates, pathToReg,
  slice = [].slice;

co = require("co");

pathToReg = require("path-to-regexp");

compose = require("./util/compose");

parseDates = require("./util/parseDates");

Request = require("./request");

Response = require("./response");

{
  http = require("http");
  chain = [require("./util/serverResponse")];
}

Application = (function() {
  var j, len, method, ref;

  function Application() {
    if (!(this instanceof Application)) {
      return new Application();
    }
    this.middleware = [];
  }

  Application.prototype.listen = function() {
    var server;
    {
      server = http.createServer(this.callback());
      return server.listen.apply(server, arguments);
    }
  };

  Application.prototype.callback = function() {
    var fn;
    fn = co.wrap(compose(chain.concat(this.middleware)));
    return function(req, res) {
      return fn.call({}, new Request(req), new Response(res))["catch"](function(err) {
        throw err;
      });
    };
  };

  Application.prototype.use = function() {
    var middlewares;
    middlewares = 1 <= arguments.length ? slice.call(arguments, 0) : [];
    this.middleware = this.middleware.concat(middlewares);
    return this;
  };

  ref = ["options", "head", "get", "put", "post", "delete"];
  for (j = 0, len = ref.length; j < len; j++) {
    method = ref[j];
    Application.prototype[method] = (function(method) {
      return function() {
        var keys, middlewares, path;
        path = arguments[0], middlewares = 2 <= arguments.length ? slice.call(arguments, 1) : [];
        keys = [];
        path = pathToReg(path, keys);
        method = method.toUpperCase();
        middlewares = compose(middlewares);
        this.use(function*(req, res, next) {
          var i, k, len1, match, name;
          if (req.method === method && (match = req.url.pathname.match(path.regex))) {
            req.params = {};
            for (i = k = 0, len1 = keys.length; k < len1; i = ++k) {
              name = keys[i].name;
              req.params[name] = match[i + 1];
            }
            return (yield middlewares.call(this, req, res, next));
          } else {
            return (yield next);
          }
        });
        return this;
      };
    })(method);
  }

  return Application;

})();

module.exports = Application;
